# BEGIN LICENSE BLOCK
# 
# Copyright (c) 1996-2003 Jesse Vincent <jesse@bestpractical.com>
# 
# (Except where explictly superceded by other copyright notices)
# 
# This work is made available to you under the terms of Version 2 of
# the GNU General Public License. A copy of that license should have
# been provided with this software, but in any event can be snarfed
# from www.gnu.org.
# 
# This work is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# Unless otherwise specified, all modifications, corrections or
# extensions to this work which alter its source code become the
# property of Best Practical Solutions, LLC when submitted for
# inclusion in the work.
# 
# 
# END LICENSE BLOCK
#
package RT::Action::NotifyNonRecipients;
require RT::Action::SendEmail;
use Mail::Address;
use strict;
use vars qw/@ISA/;
@ISA = qw(RT::Action::SendEmail);


=head2 Prepare

Set up the relevant recipients, then call our parent.

=cut


sub Prepare {
    my $self = shift;
    $self->SetRecipients();
    $self->SUPER::Prepare();
}

# {{{ sub SetRecipients

=head2 SetRecipients

Sets the recipients of this meesage to Owner, Requestor, AdminCc, Cc or All. 
Explicitly B<does not> notify the creator of the transaction by default

=cut

sub SetRecipients {
    my $self = shift;

    my $arg = $self->Argument;

    $arg =~ s/\bAll\b/Owner,Requestor,AdminCc,Cc/;

    my ( @To, @PseudoTo, @Cc, @Bcc );


    if ( $arg =~ /\bRequestor\b/ ) {
        push ( @To, $self->TicketObj->Requestors->MemberEmailAddresses  );
    }

    if ( $arg =~ /\bCc\b/ ) {

        #If we have a To, make the Ccs, Ccs, otherwise, promote them to To
        if (@To) {
            push ( @Cc, $self->TicketObj->Cc->MemberEmailAddresses );
            push ( @Cc, $self->TicketObj->QueueObj->Cc->MemberEmailAddresses  );
        }
        else {
            push ( @Cc, $self->TicketObj->Cc->MemberEmailAddresses  );
            push ( @To, $self->TicketObj->QueueObj->Cc->MemberEmailAddresses  );
        }
    }

    if ( ( $arg =~ /\bOwner\b/ )
        && ( $self->TicketObj->OwnerObj->id != $RT::Nobody->id ) )
    {

        # If we're not sending to Ccs or requestors, 
        # then the Owner can be the To.
        if (@To) {
            push ( @Bcc, $self->TicketObj->OwnerObj->EmailAddress );
        }
        else {
            push ( @To, $self->TicketObj->OwnerObj->EmailAddress );
        }

    }

    if ( $arg =~ /\bAdminCc\b/ ) {
        push ( @Bcc, $self->TicketObj->AdminCc->MemberEmailAddresses  );
        push ( @Bcc, $self->TicketObj->QueueObj->AdminCc->MemberEmailAddresses  );
    }

    if ($RT::UseFriendlyToLine) {
        unless (@To) {
            push (
		@PseudoTo,
		sprintf($RT::FriendlyToLineFormat, $arg, $self->TicketObj->id),
	    );
        }
    }

    my $creator = $self->TransactionObj->CreatorObj->EmailAddress();
    my $msg_body = $self->TransactionObj->Attachments->First;

    # Strip the sender out of the To, Cc and AdminCc and set the 
    # recipients fields used to build the message by the superclass.
    # unless a flag is set 
    my %exclude;
    if (! $RT::NotifyActor) {
	$exclude{$creator} = 1;
	if ($msg_body) {
            my @msg_from = Mail::Address->parse($msg_body->GetHeader('From'));
            foreach my $addr (@msg_from) {
		$exclude{$addr->address} = 1;
	    }
	}
    }

    # Also strip any users already listed in the To and Cc mail fields
    if ($msg_body) {
	my @msg_to = Mail::Address->parse($msg_body->GetHeader('To'));
	my @msg_cc = Mail::Address->parse($msg_body->GetHeader('Cc'));
	foreach my $addr (@msg_to, @msg_cc) {
	    $exclude{$addr->address} = 1;
	}
    }

    # Remove the unwanted addresses.
    if (%exclude) {
        @To  = grep ( ! $exclude{$_}, @To );
        @Cc  = grep ( ! $exclude{$_}, @Cc );
        @Bcc = grep ( ! $exclude{$_}, @Bcc );
    }

    # Add OtherRecipients last, so that the RT-Send-* headers can be
    # used to override the exclusions.
    if ( $arg =~ /\bOtherRecipients\b/ ) {
        if ( $msg_body ) {
            my @cc_addresses = Mail::Address->parse($msg_body->GetHeader('RT-Send-Cc'));
            foreach my $addr (@cc_addresses) {
                  push @Cc, $addr->address;
            }

            my @bcc_addresses = Mail::Address->parse($msg_body->GetHeader('RT-Send-Bcc'));
            foreach my $addr (@bcc_addresses) {
                  push @Bcc, $addr->address;
            }
        }
    }

    @{ $self->{'To'} }       = @To;
    @{ $self->{'Cc'} }       = @Cc;
    @{ $self->{'Bcc'} }      = @Bcc;
    @{ $self->{'PseudoTo'} } = @PseudoTo;
}

# }}}

eval "require RT::Action::NotifyNonRecipients_Vendor";
die $@ if ($@ && $@ !~ qr{^Can't locate RT/Action/NotifyNonRecipients_Vendor.pm});
eval "require RT::Action::NotifyNonRecipients_Local";
die $@ if ($@ && $@ !~ qr{^Can't locate RT/Action/NotifyNonRecipients_Local.pm});

1;
